;+

function trace_invert_diffpatt,image,psfin,iindex, quiet=quiet

;NAME:
;     TRACE_INVERT_DIFFPATT
;PURPOSE:
;     Make a crude estimate of the TRACE intensities from the diffraction
;     pattern to correct areas of saturation. 
;CATEGORY:
;CALLING SEQUENCE:
;     newimage = trace_invert_diffpatt(image,psf,ss)
;INPUTS:
;     image = image to be corrected
;     psf = PSF for the diffraction pattern centered on floor(nx/2),floor(ny/2)
;     ss = indices in image to be corrected (usually indices of saturated
;          pixels) 
;OPTIONAL INPUT PARAMETERS:
;KEYWORD PARAMETERS
;     /quiet = no output
;OUTPUTS:
;     newimage = corrected image
;COMMON BLOCKS:
;SIDE EFFECTS:
;RESTRICTIONS:
;PROCEDURE:
;MODIFICATION HISTORY:
;     2000-March-04  T. Metcalf
;-

   outimage = float(image)

   if iindex[0] GE 0 then begin  ; otherwise no saturated pixels
      if NOT keyword_set(quiet) then $
         message,/info,'Estimating intensity in saturated areas'

      szimage = size(image)

      nx = szimage[1]
      ny = szimage[2]

      x0 = floor(nx/2)
      y0 = floor(ny/2)

      psf = psfin
      psf[x0,y0] = 0.   ; no central peak


      for i=0L,n_elements(iindex)-1L do begin
         x = iindex[i] mod nx
         y = iindex[i] / nx
         spsf = shift(psf,x-x0,y-y0)
         good = where(spsf GT 0. AND image GT 0.,ngood)
         good = where(spsf GT max(spsf)*.2 AND image GT 0.,ngood)
         if ngood GT 0 then begin

            for j=0L,ngood-1L do begin
               x1 = good[j] MOD nx
               y1 = good[j] / nx
               bg = image[(x1-10)>0:(x1+10)<(nx-1),(y1-10)>0:(y1+10)<(ny-1)]
               goodbg = where(bg LT 0.8*image[x1,y1] and bg GT 0.,ngoodbg)
               if ngoodbg GT 0 then bg = total(bg[goodbg])/ngoodbg $
               else bg = image[x1,y1]
               ds = ((image[x1,y1]-bg)>0.)/spsf[x1,y1]
               if j EQ 0 then desat = [ds] else desat=[desat,ds]
            endfor

            ; The image vaues will contain significant background so all
            ; the outimage values should be overestimated.  Hence use the
            ; min value assuming that this has the least background 
            ; contamination.
            positive = where(desat GT image[iindex[i]],npositive)
            if npositive GT 0 then $
               ;outimage[iindex[i]] = median(desat[positive])
               outimage[iindex[i]] = min(desat[positive])
            if NOT keyword_set(quiet) then begin
               tvscl,sqrt(outimage>0.)
               xyouts,nx/2,ny/10, $
                   strcompress(string(outimage[iindex[i]])+' '+string(i+1)+' of '+string(n_elements(iindex))),$
                   align=0.5,/device,color=!d.table_size/2
            endif
         endif
      endfor
   endif

   return,outimage

end

